@@ -1,7 +1,19 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2008, 2011-2014 Lukáš Lalinský
+# Copyright (C) 2009, 2018-2020 Philipp Wolfer
+# Copyright (C) 2012 Chad Wilson
+# Copyright (C) 2012-2013 Michael Wiencek
+# Copyright (C) 2013-2019 Laurent Monin
+# Copyright (C) 2015 Ohm Patel
+# Copyright (C) 2015 Sophist-UK
+# Copyright (C) 2016 Suhas
+# Copyright (C) 2016-2017 Wieland Hoffmann
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017 Ville Skyttä
+# Copyright (C) 2018 Bob Swift
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,11 +29,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import re
+
+from picard.version import (
+    Version,
+    VersionError,
+)
+
 
 PICARD_ORG_NAME = "MusicBrainz"
 PICARD_APP_NAME = "Picard"
-PICARD_VERSION = (2, 1, 2, 'final', 0)
+PICARD_DISPLAY_NAME = "MusicBrainz Picard"
+PICARD_APP_ID = "org.musicbrainz.Picard"
+PICARD_DESKTOP_NAME = PICARD_APP_ID + ".desktop"
+PICARD_VERSION = Version(2, 5, 6, 'final', 0)
 
 
 # optional build version
@@ -30,53 +50,24 @@ PICARD_VERSION = (2, 1, 2, 'final', 0)
 PICARD_BUILD_VERSION_STR = ""
 
 
-class VersionError(Exception):
-    pass
-
-
 def version_to_string(version, short=False):
+    """Deprecated: Use picard.version.Version.to_string instead"""
     if len(version) != 5:
         raise VersionError("Length != 5")
-    if version[3] not in ('final', 'dev'):
-        raise VersionError("Should be either 'final' or 'dev'")
-    _version = []
-    for p in version:
-        try:
-            n = int(p)
-        except ValueError:
-            n = p
-        _version.append(n)
-    version = tuple(_version)
-    if short and version[3] == 'final':
-        if version[2] == 0:
-            version_str = '%d.%d' % version[:2]
-        else:
-            version_str = '%d.%d.%d' % version[:3]
-    else:
-        version_str = '%d.%d.%d.%s%d' % version
-    return version_str
-
-
-_version_re = re.compile(r"(\d+)[._](\d+)(?:[._](\d+)[._]?(?:(dev|final)[._]?(\d+))?)?$")
+    if not isinstance(version, Version):
+        version = Version(*version)
+    return version.to_string(short=short)
 
 
 def version_from_string(version_str):
-    m = _version_re.search(version_str)
-    if m:
-        g = m.groups()
-        if g[2] is None:
-            return (int(g[0]), int(g[1]), 0, 'final', 0)
-        if g[3] is None:
-            return (int(g[0]), int(g[1]), int(g[2]), 'final', 0)
-        return (int(g[0]), int(g[1]), int(g[2]), g[3], int(g[4]))
-    raise VersionError("String '%s' does not match regex '%s'" % (version_str,
-                                                                  _version_re.pattern))
+    """Deprecated: Use picard.version.Version.from_string instead"""
+    return Version.from_string(version_str)
 
 
-PICARD_VERSION_STR = version_to_string(PICARD_VERSION)
-PICARD_VERSION_STR_SHORT = version_to_string(PICARD_VERSION, short=True)
+PICARD_VERSION_STR = PICARD_VERSION.to_string()
+PICARD_VERSION_STR_SHORT = PICARD_VERSION.to_string(short=True)
 if PICARD_BUILD_VERSION_STR:
-    __version__ = "%s_%s" % (PICARD_VERSION_STR, PICARD_BUILD_VERSION_STR)
+    __version__ = "%s+%s" % (PICARD_VERSION_STR, PICARD_BUILD_VERSION_STR)
     PICARD_FANCY_VERSION_STR = "%s (%s)" % (PICARD_VERSION_STR_SHORT,
                                             PICARD_BUILD_VERSION_STR)
 else:
@@ -86,5 +77,11 @@ else:
 # Keep those ordered
 api_versions = [
     "2.0",
-    "2.1"
+    "2.1",
+    "2.2",
+    "2.3",
+    "2.4",
+    "2.5",
 ]
+
+api_versions_tuple = [Version.from_string(v) for v in api_versions]
