@@ -1,8 +1,31 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2004 Robert Kaye
-# Copyright (C) 2006 Lukáš Lalinský
+# Copyright (C) 2006-2009, 2011-2014, 2017 Lukáš Lalinský
+# Copyright (C) 2008 Gary van der Merwe
+# Copyright (C) 2008 amckinle
+# Copyright (C) 2008-2010, 2014-2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2009 Carlin Mangar
+# Copyright (C) 2010 Andrew Barnert
+# Copyright (C) 2011-2014 Michael Wiencek
+# Copyright (C) 2011-2014, 2017-2019 Wieland Hoffmann
+# Copyright (C) 2012 Chad Wilson
+# Copyright (C) 2013 Calvin Walton
+# Copyright (C) 2013 Ionuț Ciocîrlan
+# Copyright (C) 2013 brainz34
+# Copyright (C) 2013-2014, 2017 Sophist-UK
+# Copyright (C) 2013-2015, 2017-2019 Laurent Monin
+# Copyright (C) 2016 Rahul Raturi
+# Copyright (C) 2016 Simon Legner
+# Copyright (C) 2016 Suhas
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017-2018 Vishal Choudhary
+# Copyright (C) 2018 Bob Swift
+# Copyright (C) 2018 virusMac
+# Copyright (C) 2019 Joel Lintunen
+# Copyright (C) 2020 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,11 +41,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 import argparse
+from collections import defaultdict
 from functools import partial
-from itertools import chain
 import logging
-from operator import attrgetter
 import os.path
 import platform
 import re
@@ -37,7 +60,9 @@ from PyQt5 import (
 )
 
 from picard import (
+    PICARD_APP_ID,
     PICARD_APP_NAME,
+    PICARD_DESKTOP_NAME,
     PICARD_FANCY_VERSION_STR,
     PICARD_ORG_NAME,
     acoustid,
@@ -48,6 +73,7 @@ from picard.acoustid.manager import AcoustIDManager
 from picard.album import (
     Album,
     NatAlbum,
+    run_album_post_removal_processors,
 )
 from picard.browser.browser import BrowserIntegration
 from picard.browser.filelookup import FileLookup
@@ -62,12 +88,18 @@ from picard.const import (
     USER_DIR,
     USER_PLUGIN_DIR,
 )
+from picard.const.sys import (
+    IS_FROZEN,
+    IS_HAIKU,
+    IS_MACOS,
+    IS_WIN,
+)
 from picard.dataobj import DataObject
 from picard.disc import Disc
 from picard.file import File
 from picard.formats import open_ as open_file
 from picard.i18n import setup_gettext
-from picard.plugin import PluginManager
+from picard.pluginmanager import PluginManager
 from picard.releasegroup import ReleaseGroup
 from picard.track import (
     NonAlbumTrack,
@@ -78,9 +110,11 @@ from picard.util import (
     decode_filename,
     encode_filename,
     is_hidden,
+    iter_files_from_objects,
     mbid_validate,
+    normpath,
+    process_events_iter,
     thread,
-    uniqify,
     versions,
     webbrowser2,
 )
@@ -91,8 +125,9 @@ from picard.webservice.api_helpers import (
     MBAPIHelper,
 )
 
-import picard.resources  # pylint: disable=unused-import
+import picard.resources  # noqa: F401 # pylint: disable=unused-import
 
+from picard.ui import theme
 from picard.ui.itemviews import BaseTreeView
 from picard.ui.mainwindow import MainWindow
 from picard.ui.searchdialog.album import AlbumSearchDialog
@@ -128,23 +163,10 @@ class Tagger(QtWidgets.QApplication):
 
     def __init__(self, picard_args, unparsed_args, localedir, autoupdate):
 
-        # Use the new fusion style from PyQt5 for a modern and consistent look
-        # across all OSes.
-        if sys.platform != "darwin":
-            self.setStyle('Fusion')
-
-        # Set the WM_CLASS to 'MusicBrainz-Picard' so desktop environments
-        # can use it to look up the app
-        super().__init__(['MusicBrainz-Picard'] + unparsed_args)
+        super().__init__(sys.argv)
         self.__class__.__instance = self
         config._setup(self, picard_args.config_file)
-
-        # Allow High DPI Support
-        self.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
-        self.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
-        super().setStyleSheet(
-            'QGroupBox::title { /* PICARD-1206, Qt bug workaround */ }'
-        )
+        theme.setup(self)
 
         self._cmdline_files = picard_args.FILE
         self.autoupdate_enabled = autoupdate
@@ -160,12 +182,19 @@ class Tagger(QtWidgets.QApplication):
         # It's a valid reference, but its start() method doesn't work.
         self.thread_pool = QtCore.QThreadPool(self)
 
+        # Provide a separate thread pool for operations that should not be
+        # delayed by longer background processing tasks, e.g. because the user
+        # expects instant feedback instead of waiting for a long list of
+        # operations to finish.
+        self.priority_thread_pool = QtCore.QThreadPool(self)
+        self.priority_thread_pool.setMaxThreadCount(1)
+
         # Use a separate thread pool for file saving, with a thread count of 1,
         # to avoid race conditions in File._save_and_rename.
         self.save_thread_pool = QtCore.QThreadPool(self)
         self.save_thread_pool.setMaxThreadCount(1)
 
-        if not sys.platform == "win32":
+        if not IS_WIN:
             # Set up signal handling
             # It's not possible to call all available functions from signal
             # handlers, therefore we need to set up a QSocketNotifier to listen
@@ -188,7 +217,7 @@ class Tagger(QtWidgets.QApplication):
             signal.signal(signal.SIGINT, self.signal)
             signal.signal(signal.SIGTERM, self.signal)
 
-        if sys.platform == "darwin":
+        if IS_MACOS:
             # On macOS it is not common that the global menu shows icons
             self.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus)
 
@@ -212,34 +241,35 @@ class Tagger(QtWidgets.QApplication):
         # translated
         setup_gettext(localedir, config.setting["ui_language"], log.debug)
 
-        upgrade_config()
+        upgrade_config(config.config)
 
         self.webservice = WebService()
         self.mb_api = MBAPIHelper(self.webservice)
-        self.acoustid_api = AcoustIdAPIHelper(self.webservice)
 
         load_user_collections()
 
         # Initialize fingerprinting
-        self._acoustid = acoustid.AcoustIDClient()
+        acoustid_api = AcoustIdAPIHelper(self.webservice)
+        self._acoustid = acoustid.AcoustIDClient(acoustid_api)
         self._acoustid.init()
+        self.acoustidmanager = AcoustIDManager(acoustid_api)
 
         # Load plugins
         self.pluginmanager = PluginManager()
         if not self._no_plugins:
-            if hasattr(sys, "frozen"):
-                self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(sys.argv[0]), "plugins"))
+            if IS_FROZEN:
+                self.pluginmanager.load_plugins_from_directory(os.path.join(os.path.dirname(sys.argv[0]), "plugins"))
             else:
                 mydir = os.path.dirname(os.path.abspath(__file__))
-                self.pluginmanager.load_plugindir(os.path.join(mydir, "plugins"))
+                self.pluginmanager.load_plugins_from_directory(os.path.join(mydir, "plugins"))
 
             if not os.path.exists(USER_PLUGIN_DIR):
                 os.makedirs(USER_PLUGIN_DIR)
-            self.pluginmanager.load_plugindir(USER_PLUGIN_DIR)
+            self.pluginmanager.load_plugins_from_directory(USER_PLUGIN_DIR)
 
-        self.acoustidmanager = AcoustIDManager()
         self.browser_integration = BrowserIntegration()
 
+        self._pending_files_count = 0
         self.files = {}
         self.clusters = ClusterList()
         self.albums = {}
@@ -247,7 +277,7 @@ class Tagger(QtWidgets.QApplication):
         self.mbid_redirects = {}
         self.unclustered_files = UnclusteredFiles()
         self.nats = None
-        self.window = MainWindow()
+        self.window = MainWindow(disable_player=picard_args.no_player)
         self.exit_cleanup = []
         self.stopping = False
 
@@ -272,16 +302,20 @@ class Tagger(QtWidgets.QApplication):
         webbrowser2.open(authorization_url)
         if not parent:
             parent = self.window
-        authorization_code, ok = QtWidgets.QInputDialog.getText(parent,
-            _("MusicBrainz Account"), _("Authorization code:"))
-        if ok:
+        dialog = QtWidgets.QInputDialog(parent)
+        dialog.setWindowModality(QtCore.Qt.WindowModal)
+        dialog.setWindowTitle(_("MusicBrainz Account"))
+        dialog.setLabelText(_("Authorization code:"))
+        status = dialog.exec_()
+        if status == QtWidgets.QDialog.Accepted:
+            authorization_code = dialog.textValue()
             self.webservice.oauth_manager.exchange_authorization_code(
                 authorization_code, scopes,
                 partial(self.on_mb_authorization_finished, callback))
         else:
             callback(False)
 
-    def on_mb_authorization_finished(self, callback, successful):
+    def on_mb_authorization_finished(self, callback, successful=False):
         if successful:
             self.webservice.oauth_manager.fetch_username(
                 partial(self.on_mb_login_finished, callback))
@@ -302,11 +336,7 @@ class Tagger(QtWidgets.QApplication):
         """Move `files` to tracks on album `albumid`."""
         if album is None:
             album = self.load_album(albumid)
-        if album.loaded:
-            album.match_files(files)
-        else:
-            for file in list(files):
-                file.move(album.unmatched_files)
+        album.match_files(files)
 
     def move_file_to_album(self, file, albumid):
         """Move `file` to a track on album `albumid`."""
@@ -315,14 +345,15 @@ class Tagger(QtWidgets.QApplication):
     def move_file_to_track(self, file, albumid, recordingid):
         """Move `file` to recording `recordingid` on album `albumid`."""
         album = self.load_album(albumid)
-        file.move(album.unmatched_files)
-        album.run_when_loaded(partial(album.match_file, file, recordingid))
+        file.match_recordingid = recordingid
+        album.match_files([file])
 
     def create_nats(self):
         if self.nats is None:
             self.nats = NatAlbum()
             self.albums["NATS"] = self.nats
             self.album_added.emit(self.nats)
+            self.nats.item.setExpanded(True)
         return self.nats
 
     def move_file_to_nat(self, file, recordingid, node=None):
@@ -341,6 +372,7 @@ class Tagger(QtWidgets.QApplication):
         self._acoustid.done()
         self.thread_pool.waitForDone()
         self.save_thread_pool.waitForDone()
+        self.priority_thread_pool.waitForDone()
         self.browser_integration.stop()
         self.webservice.stop()
         self.run_cleanup()
@@ -348,14 +380,8 @@ class Tagger(QtWidgets.QApplication):
 
     def _run_init(self):
         if self._cmdline_files:
-            files = []
-            for file in self._cmdline_files:
-                if os.path.isdir(file):
-                    self.add_directory(decode_filename(file))
-                else:
-                    files.append(decode_filename(file))
-            if files:
-                self.add_files(files)
+            files = [decode_filename(f) for f in self._cmdline_files]
+            self.add_paths(files)
             del self._cmdline_files
 
     def run(self):
@@ -371,59 +397,122 @@ class Tagger(QtWidgets.QApplication):
         if isinstance(event, thread.ProxyToMainEvent):
             event.run()
         elif event.type() == QtCore.QEvent.FileOpen:
-            self.add_files([event.file()])
+            file = event.file()
+            self.add_paths([file])
+            if IS_HAIKU:
+                self.bring_tagger_front()
             # We should just return True here, except that seems to
             # cause the event's sender to get a -9874 error, so
             # apparently there's some magic inside QFileOpenEvent...
             return 1
         return super().event(event)
 
-    def _file_loaded(self, file, target=None):
-        if file is not None and not file.has_error():
-            recordingid = file.metadata.getall('musicbrainz_recordingid')[0] \
-                if 'musicbrainz_recordingid' in file.metadata else ''
-            if target is not None:
-                self.move_files([file], target)
-            elif not config.setting["ignore_file_mbids"]:
-                albumid = file.metadata.getall('musicbrainz_albumid')[0] \
-                    if 'musicbrainz_albumid' in file.metadata else ''
-                if mbid_validate(albumid):
-                    if mbid_validate(recordingid):
-                        self.move_file_to_track(file, albumid, recordingid)
-                    else:
-                        self.move_file_to_album(file, albumid)
-                elif mbid_validate(recordingid):
-                    self.move_file_to_nat(file, recordingid)
-                elif config.setting['analyze_new_files'] and file.can_analyze():
-                    self.analyze([file])
-            elif config.setting['analyze_new_files'] and file.can_analyze():
-                self.analyze([file])
+    def _file_loaded(self, file, target=None, remove_file=False):
+        self._pending_files_count -= 1
+        if self._pending_files_count == 0:
+            self.window.set_sorting(True)
 
-    def move_files(self, files, target):
+        if remove_file:
+            file.remove()
+            return
+
+        if file is None:
+            return
+
+        if file.has_error():
+            self.unclustered_files.add_file(file)
+            return
+
+        if not config.setting["ignore_file_mbids"]:
+            recordingid = file.metadata.getall('musicbrainz_recordingid')
+            recordingid = recordingid[0] if recordingid else ''
+            is_valid_recordingid = mbid_validate(recordingid)
+
+            albumid = file.metadata.getall('musicbrainz_albumid')
+            albumid = albumid[0] if albumid else ''
+            is_valid_albumid = mbid_validate(albumid)
+
+            if is_valid_albumid and is_valid_recordingid:
+                log.debug("%r has release (%s) and recording (%s) MBIDs, moving to track...",
+                          file, albumid, recordingid)
+                self.move_file_to_track(file, albumid, recordingid)
+                return
+
+            if is_valid_albumid:
+                log.debug("%r has only release MBID (%s), moving to album...",
+                          file, albumid)
+                self.move_file_to_album(file, albumid)
+                return
+
+            if is_valid_recordingid:
+                log.debug("%r has only recording MBID (%s), moving to non-album track...",
+                          file, recordingid)
+                self.move_file_to_nat(file, recordingid)
+                return
+
+        self.move_file(file, target)
+
+        # fallback on analyze if nothing else worked
+        if config.setting['analyze_new_files'] and file.can_analyze():
+            log.debug("Trying to analyze %r ...", file)
+            self.analyze([file])
+
+    def move_file(self, file, target):
+        if target is None:
+            if not file.parent:
+                target = self.unclustered_files
+            else:
+                log.debug("Aborting move since target is invalid and file already has a parent")
+        if isinstance(target, Album):
+            self.move_files_to_album([file], album=target)
+        elif isinstance(target, File) and target.parent:
+            file.move(target.parent)
+        # To be able to move a file to it target must implement add_file(file)
+        elif hasattr(target, 'add_file'):
+            file.move(target)
+        # Ensure a file always has a parent so it shows up in UI
+        elif not file.parent:
+            file.move(self.unclustered_files)
+
+    def move_files(self, files, target, move_to_multi_tracks=True):
         if target is None:
             log.debug("Aborting move since target is invalid")
             return
-        if isinstance(target, (Track, Cluster)):
-            for file in files:
+        self.window.set_sorting(False)
+        if isinstance(target, Cluster):
+            for file in process_events_iter(files):
                 file.move(target)
+        elif isinstance(target, Track):
+            album = target.album
+            for file in process_events_iter(files):
+                file.move(target)
+                if move_to_multi_tracks:  # Assign next file to following track
+                    target = album.get_next_track(target) or album.unmatched_files
         elif isinstance(target, File):
-            for file in files:
+            for file in process_events_iter(files):
                 file.move(target.parent)
         elif isinstance(target, Album):
             self.move_files_to_album(files, album=target)
         elif isinstance(target, ClusterList):
+            for file in process_events_iter(files):
+                if isinstance(file.parent, Track):
+                    file.parent.remove_file(file)
             self.cluster(files)
+        self.window.set_sorting(True)
 
     def add_files(self, filenames, target=None):
         """Add files to the tagger."""
         ignoreregex = None
         pattern = config.setting['ignore_regex']
         if pattern:
-            ignoreregex = re.compile(pattern)
+            try:
+                ignoreregex = re.compile(pattern)
+            except re.error as e:
+                log.error("Failed evaluating regular expression for ignore_regex: %s", e)
         ignore_hidden = config.setting["ignore_hidden_files"]
         new_files = []
         for filename in filenames:
-            filename = os.path.normpath(os.path.realpath(filename))
+            filename = normpath(filename)
             if ignore_hidden and is_hidden(filename):
                 log.debug("File ignored (hidden): %r" % (filename))
                 continue
@@ -439,85 +528,46 @@ class Tagger(QtWidgets.QApplication):
                 if file:
                     self.files[filename] = file
                     new_files.append(file)
+                QtCore.QCoreApplication.processEvents()
         if new_files:
             log.debug("Adding files %r", new_files)
             new_files.sort(key=lambda x: x.filename)
-            if target is None or target is self.unclustered_files:
-                self.unclustered_files.add_files(new_files)
-                target = None
-            for file in new_files:
+            self.window.set_sorting(False)
+            self._pending_files_count += len(new_files)
+            for i, file in enumerate(new_files):
                 file.load(partial(self._file_loaded, target=target))
+                # Calling processEvents helps processing the _file_loaded
+                # callbacks in between, which keeps the UI more responsive.
+                # Avoid calling it to often to not slow down the loading to much
+                # Using an uneven number to have the unclustered file counter
+                # not look stuck in certain digits.
+                if i % 17 == 0:
+                    QtCore.QCoreApplication.processEvents()
 
-    def add_directory(self, path):
-        if config.setting['recursively_add_files']:
-            self._add_directory_recursive(path)
-        else:
-            self._add_directory_non_recursive(path)
-
-    def _add_directory_recursive(self, path):
-        ignore_hidden = config.setting["ignore_hidden_files"]
-        walk = os.walk(path)
-
-        def get_files():
+    @staticmethod
+    def _scan_paths_recursive(paths, recursive, ignore_hidden):
+        local_paths = list(paths)
+        while local_paths:
+            current_path = local_paths.pop(0)
             try:
-                root, dirs, files = next(walk)
-                if ignore_hidden:
-                    dirs[:] = [d for d in dirs if not is_hidden(os.path.join(root, d))]
-            except StopIteration:
-                return None
-            else:
-                number_of_files = len(files)
-                if number_of_files:
-                    mparms = {
-                        'count': number_of_files,
-                        'directory': root,
-                    }
-                    log.debug("Adding %(count)d files from '%(directory)r'" %
-                              mparms)
-                    self.window.set_statusbar_message(
-                        ngettext(
-                            "Adding %(count)d file from '%(directory)s' ...",
-                            "Adding %(count)d files from '%(directory)s' ...",
-                            number_of_files),
-                        mparms,
-                        translate=None,
-                        echo=None
-                    )
-                return (os.path.join(root, f) for f in files)
+                if os.path.isdir(current_path):
+                    for entry in os.scandir(current_path):
+                        if ignore_hidden and is_hidden(entry.path):
+                            continue
+                        if recursive and entry.is_dir():
+                            local_paths.append(entry.path)
+                        else:
+                            yield entry.path
+                else:
+                    yield current_path
+            except OSError as err:
+                log.warning(err)
 
-        def process(result=None, error=None):
-            if result:
-                if error is None:
-                    self.add_files(result)
-                thread.run_task(get_files, process)
-
-        process(True, False)
-
-    def _add_directory_non_recursive(self, path):
-        files = []
-        for f in os.listdir(path):
-            listing = os.path.join(path, f)
-            if os.path.isfile(listing):
-                files.append(listing)
-        number_of_files = len(files)
-        if number_of_files:
-            mparms = {
-                'count': number_of_files,
-                'directory': path,
-            }
-            log.debug("Adding %(count)d files from '%(directory)r'" %
-                      mparms)
-            self.window.set_statusbar_message(
-                ngettext(
-                    "Adding %(count)d file from '%(directory)s' ...",
-                    "Adding %(count)d files from '%(directory)s' ...",
-                    number_of_files),
-                mparms,
-                translate=None,
-                echo=None
-            )
-            # Function call only if files exist
-            self.add_files(files)
+    def add_paths(self, paths, target=None):
+        files = self._scan_paths_recursive(paths,
+                            config.setting['recursively_add_files'],
+                            config.setting["ignore_hidden_files"])
+        self.add_files(files, target=target)
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
@@ -527,7 +577,7 @@ class Tagger(QtWidgets.QApplication):
 
     def copy_files(self, objects):
         mimeData = QtCore.QMimeData()
-        mimeData.setUrls([QtCore.QUrl.fromLocalFile(f.filename) for f in (self.get_files_from_objects(objects))])
+        mimeData.setUrls([QtCore.QUrl.fromLocalFile(f.filename) for f in iter_files_from_objects(objects)])
         self.clipboard().setMimeData(mimeData)
 
     def paste_files(self, target):
@@ -590,13 +640,15 @@ class Tagger(QtWidgets.QApplication):
                 item.filename if isinstance(item, File) else '')
 
     def get_files_from_objects(self, objects, save=False):
-        """Return list of files from list of albums, clusters, tracks or files."""
-        return uniqify(chain(*[obj.iterfiles(save) for obj in objects]))
+        """Return list of unique files from list of albums, clusters, tracks or files.
+
+        Note: Consider using picard.util.iter_files_from_objects instead, which returns an iterator.
+        """
+        return list(iter_files_from_objects(objects, save=save))
 
     def save(self, objects):
         """Save the specified objects."""
-        files = self.get_files_from_objects(objects, save=True)
-        for file in files:
+        for file in iter_files_from_objects(objects, save=True):
             file.save()
 
     def load_album(self, album_id, discid=None):
@@ -604,6 +656,7 @@ class Tagger(QtWidgets.QApplication):
         album = self.albums.get(album_id)
         if album:
             log.debug("Album %s already loaded.", album_id)
+            album.add_discid(discid)
             return album
         album = Album(album_id, discid=discid)
         self.albums[album_id] = album
@@ -643,18 +696,35 @@ class Tagger(QtWidgets.QApplication):
                 self._acoustid.stop_analyze(file)
                 del self.files[file.filename]
                 file.remove(from_parent)
+        self.tagger_stats_changed.emit()
 
     def remove_album(self, album):
         """Remove the specified album."""
         log.debug("Removing %r", album)
+        if album.id not in self.albums:
+            return
         album.stop_loading()
-        self.remove_files(self.get_files_from_objects([album]))
+        self.remove_files(list(album.iterfiles()))
         del self.albums[album.id]
         if album.release_group:
             album.release_group.remove_album(album.id)
         if album == self.nats:
             self.nats = None
         self.album_removed.emit(album)
+        run_album_post_removal_processors(album)
+        self.tagger_stats_changed.emit()
+
+    def remove_nat(self, track):
+        """Remove the specified non-album track."""
+        log.debug("Removing %r", track)
+        self.remove_files(list(track.iterfiles()))
+        if not self.nats:
+            return
+        self.nats.tracks.remove(track)
+        if not self.nats.tracks:
+            self.remove_album(self.nats)
+        else:
+            self.nats.update(True)
 
     def remove_cluster(self, cluster):
         """Remove the specified cluster."""
@@ -670,27 +740,30 @@ class Tagger(QtWidgets.QApplication):
     def remove(self, objects):
         """Remove the specified objects."""
         files = []
-        for obj in objects:
-            if isinstance(obj, File):
-                files.append(obj)
-            elif isinstance(obj, Track):
-                files.extend(obj.linked_files)
-            elif isinstance(obj, Album):
-                self.window.set_statusbar_message(
-                    N_("Removing album %(id)s: %(artist)s - %(album)s"),
-                    {
-                        'id': obj.id,
-                        'artist': obj.metadata['albumartist'],
-                        'album': obj.metadata['album']
-                    }
-                )
-                self.remove_album(obj)
-            elif isinstance(obj, UnclusteredFiles):
-                files.extend(list(obj.files))
-            elif isinstance(obj, Cluster):
-                self.remove_cluster(obj)
-        if files:
-            self.remove_files(files)
+        with self.window.ignore_selection_changes:
+            for obj in objects:
+                if isinstance(obj, File):
+                    files.append(obj)
+                elif isinstance(obj, NonAlbumTrack):
+                    self.remove_nat(obj)
+                elif isinstance(obj, Track):
+                    files.extend(obj.files)
+                elif isinstance(obj, Album):
+                    self.window.set_statusbar_message(
+                        N_("Removing album %(id)s: %(artist)s - %(album)s"),
+                        {
+                            'id': obj.id,
+                            'artist': obj.metadata['albumartist'],
+                            'album': obj.metadata['album']
+                        }
+                    )
+                    self.remove_album(obj)
+                elif isinstance(obj, UnclusteredFiles):
+                    files.extend(list(obj.files))
+                elif isinstance(obj, Cluster):
+                    self.remove_cluster(obj)
+            if files:
+                self.remove_files(files)
 
     def _lookup_disc(self, disc, result=None, error=None):
         self.restore_cursor()
@@ -703,7 +776,7 @@ class Tagger(QtWidgets.QApplication):
     def lookup_cd(self, action):
         """Reads CD from the selected drive and tries to lookup the DiscID on MusicBrainz."""
         if isinstance(action, QtWidgets.QAction):
-            device = action.text()
+            device = action.data()
         elif config.setting["cd_lookup_device"] != '':
             device = config.setting["cd_lookup_device"].split(",", 1)[0]
         else:
@@ -723,11 +796,24 @@ class Tagger(QtWidgets.QApplication):
 
     def analyze(self, objs):
         """Analyze the file(s)."""
-        files = self.get_files_from_objects(objs)
-        for file in files:
+        if not self.use_acoustid:
+            return
+        for file in iter_files_from_objects(objs):
+            if file.can_analyze():
+                file.set_pending()
+                self._acoustid.analyze(file, partial(file._lookup_finished, File.LOOKUP_ACOUSTID))
+
+    def generate_fingerprints(self, objs):
+        """Generate the fingerprints without matching the files."""
+        if not self.use_acoustid:
+            return
+
+        def finished(file, result):
+            file.clear_pending()
+
+        for file in iter_files_from_objects(objs):
             file.set_pending()
-            if self.use_acoustid:
-                self._acoustid.analyze(file, partial(file._lookup_finished, 'acoustid'))
+            self._acoustid.fingerprint(file, partial(finished, file))
 
     # =======================================================================
     #  Metadata-based lookups
@@ -749,11 +835,16 @@ class Tagger(QtWidgets.QApplication):
             files = list(self.unclustered_files.files)
         else:
             files = self.get_files_from_objects(objs)
-        for name, artist, files in Cluster.cluster(files, 1.0):
-            QtCore.QCoreApplication.processEvents()
-            cluster = self.load_cluster(name, artist)
-            for file in sorted(files, key=attrgetter('discnumber', 'tracknumber', 'base_filename')):
-                file.move(cluster)
+
+        with self.window.ignore_selection_changes:
+            self.window.set_sorting(False)
+            cluster_files = defaultdict(list)
+            for name, artist, files in Cluster.cluster(files, 1.0, self):
+                cluster = self.load_cluster(name, artist)
+                cluster_files[cluster].extend(files)
+            for cluster, files in process_events_iter(cluster_files.items()):
+                cluster.add_files(files)
+            self.window.set_sorting(True)
 
     def load_cluster(self, name, artist):
         for cluster in self.clusters:
@@ -817,11 +908,21 @@ def process_picard_args():
     parser = argparse.ArgumentParser(
         epilog="If one of the filenames begins with a hyphen, use -- to separate the options from the filenames."
     )
+    # Qt default arguments. Parse them so Picard does not interpret the
+    # arguments as file names to load.
+    parser.add_argument("-style", nargs=1, help=argparse.SUPPRESS)
+    parser.add_argument("-stylesheet", nargs=1, help=argparse.SUPPRESS)
+    # Same for default X arguments
+    parser.add_argument("-display", nargs=1, help=argparse.SUPPRESS)
+
+    # Picard specific arguments
     parser.add_argument("-c", "--config-file", action='store',
                         default=None,
                         help="location of the configuration file")
     parser.add_argument("-d", "--debug", action='store_true',
                         help="enable debug-level logging")
+    parser.add_argument("-M", "--no-player", action='store_true',
+                        help="disable built-in media player")
     parser.add_argument("-N", "--no-restore", action='store_true',
                         help="do not restore positions and/or sizes")
     parser.add_argument("-P", "--no-plugins", action='store_true',
@@ -839,6 +940,20 @@ def main(localedir=None, autoupdate=True):
     # Some libs (ie. Phonon) require those to be set
     QtWidgets.QApplication.setApplicationName(PICARD_APP_NAME)
     QtWidgets.QApplication.setOrganizationName(PICARD_ORG_NAME)
+    QtWidgets.QApplication.setDesktopFileName(PICARD_DESKTOP_NAME)
+
+    # Allow High DPI Support
+    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
+    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
+    # HighDpiScaleFactorRoundingPolicy is available since Qt 5.14. This is
+    # required to support fractional scaling on Windows properly.
+    # It causes issues without scaling on Linux, see https://tickets.metabrainz.org/browse/PICARD-1948
+    if IS_WIN and hasattr(QtGui.QGuiApplication, 'setHighDpiScaleFactorRoundingPolicy'):
+        QtGui.QGuiApplication.setHighDpiScaleFactorRoundingPolicy(
+            QtCore.Qt.HighDpiScaleFactorRoundingPolicy.PassThrough)
+
+    # Enable mnemonics on all platforms, even macOS
+    QtGui.qt_set_sequence_auto_mnemonic(True)
 
     signal.signal(signal.SIGINT, signal.SIG_DFL)
 
@@ -848,6 +963,13 @@ def main(localedir=None, autoupdate=True):
     if picard_args.long_version:
         return longversion()
 
+    try:
+        from PyQt5.QtDBus import QDBusConnection
+        dbus = QDBusConnection.sessionBus()
+        dbus.registerService(PICARD_APP_ID)
+    except ImportError:
+        pass
+
     tagger = Tagger(picard_args, unparsed_args, localedir, autoupdate)
 
     # Initialize Qt default translations
@@ -858,7 +980,7 @@ def main(localedir=None, autoupdate=True):
     if translator.load(locale, "qtbase_", directory=translation_path):
         tagger.installTranslator(translator)
     else:
-        log.warning('Error loading Qt locale %s', locale.name())
+        log.debug('Qt locale %s not available', locale.name())
 
     tagger.startTimer(1000)
     sys.exit(tagger.run())
