@@ -1,7 +1,18 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006-2007 Lukáš Lalinský
+#
+# Copyright (C) 2006-2008 Lukáš Lalinský
+# Copyright (C) 2008 Hendrik van Antwerpen
+# Copyright (C) 2008-2009, 2019-2020 Philipp Wolfer
+# Copyright (C) 2011 Andrew Barnert
+# Copyright (C) 2012-2013 Michael Wiencek
+# Copyright (C) 2013 Wieland Hoffmann
+# Copyright (C) 2013, 2017 Sophist-UK
+# Copyright (C) 2013, 2018-2019 Laurent Monin
+# Copyright (C) 2015 Jeroen Kromwijk
+# Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2018 Vishal Choudhary
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,45 +26,67 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-#
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
 
 import os
-import sys
-from PyQt4 import QtCore, QtGui
+
+from PyQt5 import (
+    QtCore,
+    QtWidgets,
+)
+from PyQt5.QtCore import QStandardPaths
+
 from picard import config
+from picard.const.sys import IS_MACOS
 from picard.formats import supported_formats
 from picard.util import find_existing_path
 
 
-class FileBrowser(QtGui.QTreeView):
+_default_current_browser_path = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
+
+
+class FileBrowser(QtWidgets.QTreeView):
 
     options = [
-        config.TextOption("persist", "current_browser_path", ""),
+        config.TextOption("persist", "current_browser_path", _default_current_browser_path),
         config.BoolOption("persist", "show_hidden_files", False),
     ]
 
     def __init__(self, parent):
-        QtGui.QTreeView.__init__(self, parent)
-        self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+        super().__init__(parent)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
         self.setDragEnabled(True)
-        self.move_files_here_action = QtGui.QAction(_("&Move Tagged Files Here"), self)
+        self.load_selected_files_action = QtWidgets.QAction(_("&Load selected files"), self)
+        self.load_selected_files_action.triggered.connect(self.load_selected_files)
+        self.addAction(self.load_selected_files_action)
+        self.move_files_here_action = QtWidgets.QAction(_("&Move tagged files here"), self)
         self.move_files_here_action.triggered.connect(self.move_files_here)
         self.addAction(self.move_files_here_action)
-        self.toggle_hidden_action = QtGui.QAction(_("Show &Hidden Files"), self)
+        self.toggle_hidden_action = QtWidgets.QAction(_("Show &hidden files"), self)
         self.toggle_hidden_action.setCheckable(True)
         self.toggle_hidden_action.setChecked(config.persist["show_hidden_files"])
         self.toggle_hidden_action.toggled.connect(self.show_hidden)
         self.addAction(self.toggle_hidden_action)
-        self.set_as_starting_directory_action = QtGui.QAction(_("&Set as starting directory"), self)
+        self.set_as_starting_directory_action = QtWidgets.QAction(_("&Set as starting directory"), self)
         self.set_as_starting_directory_action.triggered.connect(self.set_as_starting_directory)
         self.addAction(self.set_as_starting_directory_action)
-        self.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
+        self.doubleClicked.connect(self.load_file_for_item)
         self.focused = False
         self._set_model()
 
+    def contextMenuEvent(self, event):
+        menu = QtWidgets.QMenu(self)
+        menu.addAction(self.load_selected_files_action)
+        menu.addSeparator()
+        menu.addAction(self.move_files_here_action)
+        menu.addAction(self.toggle_hidden_action)
+        menu.addAction(self.set_as_starting_directory_action)
+        menu.exec_(event.globalPos())
+        event.accept()
+
     def _set_model(self):
-        self.model = QtGui.QFileSystemModel()
+        self.model = QtWidgets.QFileSystemModel()
         self.model.layoutChanged.connect(self._layout_changed)
         self.model.setRootPath("")
         self._set_model_filter()
@@ -65,21 +98,21 @@ class FileBrowser(QtGui.QTreeView):
         self.model.setNameFilterDisables(False)
         self.model.sort(0, QtCore.Qt.AscendingOrder)
         self.setModel(self.model)
-        if sys.platform == "darwin":
+        if IS_MACOS:
             self.setRootIndex(self.model.index("/Volumes"))
         header = self.header()
         header.hideSection(1)
         header.hideSection(2)
         header.hideSection(3)
-        header.setResizeMode(QtGui.QHeaderView.ResizeToContents)
+        header.setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
         header.setStretchLastSection(False)
         header.setVisible(False)
 
     def _set_model_filter(self):
-        filter = QtCore.QDir.AllDirs | QtCore.QDir.Files | QtCore.QDir.Drives | QtCore.QDir.NoDotAndDotDot
+        model_filter = QtCore.QDir.AllDirs | QtCore.QDir.Files | QtCore.QDir.Drives | QtCore.QDir.NoDotAndDotDot
         if config.persist["show_hidden_files"]:
-            filter |= QtCore.QDir.Hidden
-        self.model.setFilter(filter)
+            model_filter |= QtCore.QDir.Hidden
+        self.model.setFilter(model_filter)
 
     def _layout_changed(self):
         def scroll():
@@ -93,15 +126,31 @@ class FileBrowser(QtGui.QTreeView):
             self.scrollTo(self.currentIndex())
         QtCore.QTimer.singleShot(0, scroll)
 
+    def scrollTo(self, index, scrolltype=QtWidgets.QAbstractItemView.EnsureVisible):
+        # QTreeView.scrollTo resets the horizontal scroll position to 0.
+        # Reimplemented to instead scroll to horizontal parent position or keep previous position.
+        if index and config.setting['filebrowser_horizontal_autoscroll']:
+            level = -1
+            parent = index.parent()
+            root = self.rootIndex()
+            while parent.isValid() and parent != root:
+                parent = parent.parent()
+                level += 1
+            pos_x = max(self.indentation() * level, 0)
+        else:
+            pos_x = self.horizontalScrollBar().value()
+        super().scrollTo(index, scrolltype)
+        self.horizontalScrollBar().setValue(pos_x)
+
     def mousePressEvent(self, event):
+        super().mousePressEvent(event)
         index = self.indexAt(event.pos())
         if index.isValid():
-            self.selectionModel().setCurrentIndex(index, QtGui.QItemSelectionModel.NoUpdate)
-        QtGui.QTreeView.mousePressEvent(self, event)
+            self.selectionModel().setCurrentIndex(index, QtCore.QItemSelectionModel.NoUpdate)
 
     def focusInEvent(self, event):
         self.focused = True
-        QtGui.QTreeView.focusInEvent(self, event)
+        super().focusInEvent(event)
 
     def show_hidden(self, state):
         config.persist["show_hidden_files"] = state
@@ -111,7 +160,7 @@ class FileBrowser(QtGui.QTreeView):
         indexes = self.selectedIndexes()
         if indexes:
             path = self.model.filePath(indexes[0])
-            config.persist["current_browser_path"] = os.path.normpath(unicode(path))
+            config.persist["current_browser_path"] = os.path.normpath(path)
 
     def restore_state(self):
         pass
@@ -119,22 +168,35 @@ class FileBrowser(QtGui.QTreeView):
     def _restore_state(self):
         if config.setting["starting_directory"]:
             path = config.setting["starting_directory_path"]
-            scrolltype = QtGui.QAbstractItemView.PositionAtTop
+            scrolltype = QtWidgets.QAbstractItemView.PositionAtTop
         else:
             path = config.persist["current_browser_path"]
-            scrolltype = QtGui.QAbstractItemView.PositionAtCenter
+            scrolltype = QtWidgets.QAbstractItemView.PositionAtCenter
         if path:
-            index = self.model.index(find_existing_path(unicode(path)))
+            index = self.model.index(find_existing_path(path))
             self.setCurrentIndex(index)
             self.expand(index)
             self.scrollTo(index, scrolltype)
 
     def _get_destination_from_path(self, path):
-        destination = os.path.normpath(unicode(path))
+        destination = os.path.normpath(path)
         if not os.path.isdir(destination):
             destination = os.path.dirname(destination)
         return destination
 
+    def load_file_for_item(self, index):
+        if not self.model.isDir(index):
+            QtCore.QObject.tagger.add_paths([
+                self.model.filePath(index)
+            ])
+
+    def load_selected_files(self):
+        indexes = self.selectedIndexes()
+        if not indexes:
+            return
+        paths = set(self.model.filePath(index) for index in indexes)
+        QtCore.QObject.tagger.add_paths(paths)
+
     def move_files_here(self):
         indexes = self.selectedIndexes()
         if not indexes:
