@@ -27,6 +27,7 @@
 # Copyright (C) 2018 Vishal Choudhary
 # Copyright (C) 2018 virusMac
 # Copyright (C) 2019 Timur Enikeev
+# Copyright (C) 2020 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -60,15 +61,23 @@ from picard import (
     log,
 )
 from picard.album import Album
-from picard.cluster import Cluster
+from picard.cluster import (
+    Cluster,
+    FileList,
+)
 from picard.const import PROGRAM_UPDATE_LEVELS
-from picard.const.sys import IS_MACOS
+from picard.const.sys import (
+    IS_MACOS,
+    IS_WIN,
+)
 from picard.file import File
 from picard.formats import supported_formats
 from picard.plugin import ExtensionPoint
 from picard.track import Track
 from picard.util import (
     icontheme,
+    iter_files_from_objects,
+    iter_unique,
     restore_method,
     thread,
     throttle,
@@ -80,6 +89,7 @@ from picard.util.cdrom import (
 )
 
 from picard.ui import PreserveGeometry
+from picard.ui.aboutdialog import AboutDialog
 from picard.ui.coverartbox import CoverArtBox
 from picard.ui.filebrowser import FileBrowser
 from picard.ui.infodialog import (
@@ -100,7 +110,6 @@ from picard.ui.passworddialog import (
     PasswordDialog,
     ProxyDialog,
 )
-from picard.ui.playertoolbar import Player
 from picard.ui.searchdialog.album import AlbumSearchDialog
 from picard.ui.searchdialog.track import TrackSearchDialog
 from picard.ui.statusindicator import DesktopStatusIndicator
@@ -118,6 +127,35 @@ def register_ui_init(function):
     ui_init.register(function.__module__, function)
 
 
+class IgnoreSelectionContext:
+    """Context manager for holding a boolean value, indicating whether selection changes are performed or not.
+    By default the context resolves to False. If entered it is True. This allows
+    to temporarily set a state on a block of code like:
+
+        ignore_changes = IgnoreSelectionContext()
+        # Initially ignore_changes is True
+        with ignore_changes:
+            # Perform some tasks with ignore_changes now being True
+            ...
+        # ignore_changes is False again
+    """
+
+    def __init__(self, onexit=None):
+        self._entered = 0
+        self._onexit = onexit
+
+    def __enter__(self):
+        self._entered += 1
+
+    def __exit__(self, type, value, tb):
+        self._entered -= 1
+        if self._onexit:
+            self._onexit()
+
+    def __bool__(self):
+        return self._entered > 0
+
+
 class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     defaultsize = QtCore.QSize(780, 560)
@@ -136,17 +174,19 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config.IntOption("persist", "mediaplayer_volume", 50),
     ]
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, disable_player=False):
         super().__init__(parent)
         self.selected_objects = []
-        self.ignore_selection_changes = False
+        self.ignore_selection_changes = IgnoreSelectionContext(self.update_selection)
         self.toolbar = None
         self.player = None
         self.status_indicators = []
-        player = Player(self)
-        if player.available:
-            self.player = player
-            self.player.error.connect(self._on_player_error)
+        if not disable_player:
+            from picard.ui.playertoolbar import Player
+            player = Player(self)
+            if player.available:
+                self.player = player
+                self.player.error.connect(self._on_player_error)
         self.setupUi()
 
     def setupUi(self):
@@ -173,8 +213,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.search_toolbar.setMovable(False)
 
         mainLayout = QtWidgets.QSplitter(QtCore.Qt.Vertical)
+        mainLayout.setChildrenCollapsible(False)
         mainLayout.setContentsMargins(0, 0, 0, 0)
-        mainLayout.setHandleWidth(1)
 
         self.panel = MainPanel(self, mainLayout)
         self.file_browser = FileBrowser(self.panel)
@@ -209,6 +249,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         for function in ui_init:
             function(self)
 
+    def set_processing(self, processing=True):
+        self.panel.set_processing(processing)
+
+    def set_sorting(self, sorting=True):
+        self.panel.set_sorting(sorting)
+        # self.panel.collapse_clusters(not sorting)
+
     def keyPressEvent(self, event):
         # On macOS Command+Backspace triggers the so called "Forward Delete".
         # It should be treated the same as the Delete button.
@@ -367,7 +414,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             return hasattr(obj, 'keys') and hasattr(obj, '__getitem__')
 
         echo = kwargs.get('echo', log.debug)
-        # _ is defined using builtins.__dict__, so setting it as default named argument
+        # _ is defined using builtins.__dict__, so setting it as default named argument
         # value doesn't work as expected
         translate = kwargs.get('translate', _)
         timeout = kwargs.get('timeout', 0)
@@ -391,9 +438,19 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def _on_submit_acoustid(self):
         if self.tagger.use_acoustid:
             if not config.setting["acoustid_apikey"]:
-                QtWidgets.QMessageBox.warning(self,
-                    _("Submission Error"),
-                    _("You need to configure your AcoustID API key before you can submit fingerprints."))
+                msg = QtWidgets.QMessageBox(self)
+                msg.setIcon(QtWidgets.QMessageBox.Information)
+                msg.setWindowModality(QtCore.Qt.WindowModal)
+                msg.setWindowTitle(_("AcoustID submission not configured"))
+                msg.setText(_(
+                    "You need to configure your AcoustID API key before you can submit fingerprints."))
+                open_options = QtWidgets.QPushButton(
+                    icontheme.lookup('preferences-desktop'), _("Open AcoustID options"))
+                msg.addButton(QtWidgets.QMessageBox.Cancel)
+                msg.addButton(open_options, QtWidgets.QMessageBox.YesRole)
+                msg.exec_()
+                if msg.clickedButton() == open_options:
+                    self.show_options("fingerprinting")
             else:
                 self.tagger.acoustidmanager.submit()
 
@@ -578,7 +635,10 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.enable_tag_saving_action.setChecked(not config.setting["dont_write_tags"])
         self.enable_tag_saving_action.triggered.connect(self.toggle_tag_saving)
 
-        self.tags_from_filenames_action = QtWidgets.QAction(_("Tags From &File Names..."), self)
+        self.tags_from_filenames_action = QtWidgets.QAction(icontheme.lookup('picard-tags-from-filename'), _("Tags From &File Names..."), self)
+        self.tags_from_filenames_action.setIconText(_("Parse File Names..."))
+        self.tags_from_filenames_action.setToolTip(_('Set tags based on the file names'))
+        self.tags_from_filenames_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+T")))
         self.tags_from_filenames_action.triggered.connect(self.open_tags_from_filenames)
         self.tags_from_filenames_action.setEnabled(False)
 
@@ -853,9 +913,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         tab_order(self.search_edit, self.search_button)
         # Panels
         tab_order(self.search_button, self.file_browser)
-        tab_order(self.file_browser, self.panel.views[0])
-        tab_order(self.panel.views[0], self.panel.views[1])
-        tab_order(self.panel.views[1], self.metadata_box)
+        self.panel.tab_order(tab_order, self.file_browser, self.metadata_box)
 
     def enable_submit(self, enabled):
         """Enable/disable the 'Submit fingerprints' action."""
@@ -893,12 +951,22 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         formats = []
         extensions = []
         for exts, name in supported_formats():
-            exts = ["*" + e for e in exts]
+            exts = ["*" + e.lower() for e in exts]
+            if not exts:
+                continue
+            if not IS_MACOS and not IS_WIN:
+                # Also consider upper case extensions
+                # macOS and Windows usually support case sensitive file names. Furthermore on both systems
+                # the file dialog filters list all extensions we provide, which becomes a bit long when we give the
+                # full list twice. Hence only do this trick on other operating systems.
+                exts.extend([e.upper() for e in exts])
+            exts.sort()
             formats.append("%s (%s)" % (name, " ".join(exts)))
             extensions.extend(exts)
         formats.sort()
         extensions.sort()
-        formats.insert(0, _("All Supported Formats") + " (%s)" % " ".join(extensions))
+        formats.insert(0, _("All supported formats") + " (%s)" % " ".join(extensions))
+        formats.insert(1, _("All files") + " (*)")
         files, _filter = QtWidgets.QFileDialog.getOpenFileNames(self, "", current_directory, ";;".join(formats))
         if files:
             config.persist["current_directory"] = os.path.dirname(files[0])
@@ -933,14 +1001,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                     {'directory': dir_list[0]}
                 )
 
-            for directory in dir_list:
-                self.tagger.add_directory(directory)
+            self.tagger.add_paths(dir_list)
 
     def close_active_window(self):
         self.tagger.activeWindow().close()
 
     def show_about(self):
-        return self.show_options("about")
+        return AboutDialog.show_instance(self)
 
     def show_options(self, page=None):
         return OptionsDialog.show_instance(page, self)
@@ -991,16 +1058,16 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         return QtCore.QUrl.fromLocalFile(url)
 
     def play_file(self):
-        files = self.tagger.get_files_from_objects(self.selected_objects)
-        for file in files:
+        for file in iter_files_from_objects(self.selected_objects):
             QtGui.QDesktopServices.openUrl(self._openUrl(file.filename))
 
     def _on_player_error(self, error, msg):
         self.set_statusbar_message(msg, echo=log.warning, translate=None)
 
     def open_folder(self):
-        files = self.tagger.get_files_from_objects(self.selected_objects)
-        folders = set([os.path.dirname(f.filename) for f in files])
+        folders = iter_unique(
+            os.path.dirname(f.filename) for f
+            in iter_files_from_objects(self.selected_objects))
         for folder in folders:
             QtGui.QDesktopServices.openUrl(self._openUrl(folder))
 
@@ -1025,7 +1092,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def show_more_tracks(self):
         obj = self.selected_objects[0]
         if isinstance(obj, Track):
-            obj = obj.linked_files[0]
+            obj = obj.files[0]
         dialog = TrackSearchDialog(self)
         dialog.load_similar_tracks(obj)
         dialog.exec_()
@@ -1105,7 +1172,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.tags_from_filenames_action.setEnabled(bool(files))
         self.track_search_action.setEnabled(have_objects)
 
-    def update_selection(self, objects=None):
+    def update_selection(self, objects=None, new_selection=True, drop_album_caches=False):
         if self.ignore_selection_changes:
             return
 
@@ -1116,8 +1183,6 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
         self.update_actions()
 
-        metadata = None
-        orig_metadata = None
         obj = None
 
         # Clear any existing status bar messages
@@ -1129,13 +1194,11 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if len(objects) == 1:
             obj = list(objects)[0]
             if isinstance(obj, File):
-                metadata = obj.metadata
-                orig_metadata = obj.orig_metadata
                 if obj.state == obj.ERROR:
                     msg = N_("%(filename)s (error: %(error)s)")
                     mparms = {
                         'filename': obj.filename,
-                        'error': obj.error
+                        'error': obj.errors[0] if obj.errors else ''
                     }
                 else:
                     msg = N_("%(filename)s")
@@ -1144,16 +1207,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                     }
                 self.set_statusbar_message(msg, mparms, echo=None, history=None)
             elif isinstance(obj, Track):
-                metadata = obj.metadata
                 if obj.num_linked_files == 1:
-                    file = obj.linked_files[0]
-                    orig_metadata = file.orig_metadata
-                    if file.state == File.ERROR:
+                    file = obj.files[0]
+                    if file.has_error():
                         msg = N_("%(filename)s (%(similarity)d%%) (error: %(error)s)")
                         mparms = {
                             'filename': file.filename,
                             'similarity': file.similarity * 100,
-                            'error': file.error
+                            'error': file.errors[0] if file.errors else ''
                         }
                     else:
                         msg = N_("%(filename)s (%(similarity)d%%)")
@@ -1163,17 +1224,21 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                         }
                     self.set_statusbar_message(msg, mparms, echo=None,
                                                history=None)
-            elif isinstance(obj, Album):
-                metadata = obj.metadata
-                orig_metadata = obj.orig_metadata
-            elif obj.can_show_coverart:
-                metadata = obj.metadata
+        elif new_selection:
+            # Create a temporary file list which allows changing cover art for all selected files
+            files = self.tagger.get_files_from_objects(objects)
+            obj = FileList(files)
 
-        self.metadata_box.selection_dirty = True
-        self.metadata_box.update()
-        self.cover_art_box.set_metadata(metadata, orig_metadata, obj)
+        if new_selection:
+            self.metadata_box.selection_dirty = True
+            self.cover_art_box.set_item(obj)
+        self.metadata_box.update(drop_album_caches=drop_album_caches)
         self.selection_updated.emit(objects)
 
+    def refresh_metadatabox(self):
+        self.tagger.window.metadata_box.selection_dirty = True
+        self.tagger.window.metadata_box.update()
+
     def show_cover_art(self):
         """Show/hide the cover art box."""
         if self.show_cover_art_action.isChecked():
