@@ -1,8 +1,19 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2004 Robert Kaye
-# Copyright (C) 2006 Lukáš Lalinský
+# Copyright (C) 2006-2007, 2012 Lukáš Lalinský
+# Copyright (C) 2011-2014 Michael Wiencek
+# Copyright (C) 2012 Nikolai Prokoschenko
+# Copyright (C) 2013-2014 Sophist-UK
+# Copyright (C) 2013-2014, 2017-2019 Laurent Monin
+# Copyright (C) 2015 Ohm Patel
+# Copyright (C) 2015 Wieland Hoffmann
+# Copyright (C) 2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2020 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,6 +29,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from collections import defaultdict
 from functools import partial
 
@@ -37,33 +49,30 @@ from picard.metadata import MULTI_VALUED_JOINER
 from picard.track import Track
 from picard.util import (
     format_time,
+    icontheme,
     restore_method,
     thread,
     throttle,
 )
+from picard.util.preservedtags import PreservedTags
 from picard.util.tags import display_tag_name
 
-from picard.ui.edittagdialog import EditTagDialog
-
-COMMON_TAGS = [
-    "title",
-    "artist",
-    "album",
-    "tracknumber",
-    "~length",
-    "date",
-]
+from picard.ui.colors import interface_colors
+from picard.ui.edittagdialog import (
+    EditTagDialog,
+    TagEditorDelegate,
+)
 
 
 class TagStatus:
 
-    NoChange = 1
-    Added = 2
-    Removed = 4
-    # Added | Removed = Changed
-    Changed = 6
-    Empty = 8
-    NotRemovable = 16
+    NOCHANGE = 1
+    ADDED = 2
+    REMOVED = 4
+    CHANGED = ADDED | REMOVED
+    EMPTY = 8
+    NOTREMOVABLE = 16
+    READONLY = 32
 
 
 class TagCounter(dict):
@@ -123,7 +132,7 @@ class TagDiff(object):
         else:
             return orig != new
 
-    def add(self, tag, orig_values, new_values, removable, removed=False):
+    def add(self, tag, orig_values, new_values, removable, removed=False, readonly=False):
         if orig_values:
             self.orig.add(tag, orig_values)
 
@@ -131,53 +140,54 @@ class TagDiff(object):
             self.new.add(tag, new_values)
 
         if (orig_values and not new_values) or removed:
-            self.status[tag] |= TagStatus.Removed
+            self.status[tag] |= TagStatus.REMOVED
         elif new_values and not orig_values:
-            self.status[tag] |= TagStatus.Added
+            self.status[tag] |= TagStatus.ADDED
             removable = True
         elif orig_values and new_values and self.__tag_ne(tag, orig_values, new_values):
-            self.status[tag] |= TagStatus.Changed
-        elif not (orig_values or new_values or tag in COMMON_TAGS):
-            self.status[tag] |= TagStatus.Empty
+            self.status[tag] |= TagStatus.CHANGED
+        elif not (orig_values or new_values or tag in config.setting['metadatabox_top_tags']):
+            self.status[tag] |= TagStatus.EMPTY
         else:
-            self.status[tag] |= TagStatus.NoChange
+            self.status[tag] |= TagStatus.NOCHANGE
 
         if not removable:
-            self.status[tag] |= TagStatus.NotRemovable
+            self.status[tag] |= TagStatus.NOTREMOVABLE
+
+        if readonly:
+            self.status[tag] |= TagStatus.READONLY
 
     def tag_status(self, tag):
         status = self.status[tag]
-        for s in (TagStatus.Changed, TagStatus.Added,
-                  TagStatus.Removed, TagStatus.Empty):
+        for s in (TagStatus.CHANGED, TagStatus.ADDED,
+                  TagStatus.REMOVED, TagStatus.EMPTY):
             if status & s == s:
                 return s
-        return TagStatus.NoChange
+        return TagStatus.NOCHANGE
 
 
-class PreservedTags:
+class TableTagEditorDelegate(TagEditorDelegate):
 
-    opt_name = 'preserved_tags'
+    def createEditor(self, parent, option, index):
+        editor = super().createEditor(parent, option, index)
+        if editor and isinstance(editor, QtWidgets.QPlainTextEdit):
+            table = self.parent()
+            # Set the editor to the row height, but at least 80 pixel
+            # to allow for proper multiline editing.
+            height = max(80, table.rowHeight(index.row()) - 1)
+            editor.setMinimumSize(QtCore.QSize(0, height))
+            # Resize the row so the editor fits in. Add 1 pixel, otherwise the
+            # frame gets hidden.
+            table.setRowHeight(index.row(), editor.frameSize().height() + 1)
+        return editor
 
-    def __init__(self):
-        self._tags = self._from_config()
+    def sizeHint(self, option, index):
+        # Expand the row for multiline content, but limit the maximum row height.
+        size_hint = super().sizeHint(option, index)
+        return QtCore.QSize(size_hint.width(), min(160, size_hint.height()))
 
-    def _to_config(self):
-        config.setting[self.opt_name] = ", ".join(sorted(self._tags))
-
-    def _from_config(self):
-        tags = config.setting[self.opt_name].split(',')
-        return set(filter(bool, map(str.strip, tags)))
-
-    def add(self, name):
-        self._tags.add(name)
-        self._to_config()
-
-    def discard(self, name):
-        self._tags.discard(name)
-        self._to_config()
-
-    def __contains__(self, key):
-        return key in self._tags
+    def get_tag_name(self, index):
+        return index.data(QtCore.Qt.UserRole)
 
 
 class MetadataBox(QtWidgets.QTableWidget):
@@ -187,6 +197,9 @@ class MetadataBox(QtWidgets.QTableWidget):
         config.BoolOption("persist", "show_changes_first", False)
     )
 
+    COLUMN_ORIG = 1
+    COLUMN_NEW = 2
+
     def __init__(self, parent):
         super().__init__(parent)
         self.parent = parent
@@ -204,12 +217,8 @@ class MetadataBox(QtWidgets.QTableWidget):
         self.setTabKeyNavigation(False)
         self.setStyleSheet("QTableWidget {border: none;}")
         self.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 1)
-        self.colors = {
-            TagStatus.NoChange: self.palette().color(QtGui.QPalette.Text),
-            TagStatus.Removed: QtGui.QBrush(QtGui.QColor("red")),
-            TagStatus.Added: QtGui.QBrush(QtGui.QColor("green")),
-            TagStatus.Changed: QtGui.QBrush(QtGui.QColor("darkgoldenrod"))
-        }
+        self.setItemDelegate(TableTagEditorDelegate(self))
+        self.setWordWrap(False)
         self.files = set()
         self.tracks = set()
         self.objects = set()
@@ -232,6 +241,9 @@ class MetadataBox(QtWidgets.QTableWidget):
         # TR: Keyboard shortcut for "Remove" (tag)
         self.remove_tag_shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(_("Alt+Shift+R")), self, self.remove_selected_tags)
         self.preserved_tags = PreservedTags()
+        self._single_file_album = False
+        self._single_track_album = False
+        self.tagger.clipboard().dataChanged.connect(self.update_clipboard)
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
@@ -249,6 +261,7 @@ class MetadataBox(QtWidgets.QTableWidget):
             "musicbrainz_artistid": lookup.artist_lookup,
             "musicbrainz_albumartistid": lookup.artist_lookup,
             "musicbrainz_releasegroupid": lookup.release_group_lookup,
+            "musicbrainz_discid": lookup.discid_lookup,
             "acoustid_id": lookup.acoust_lookup
         }
         return LOOKUP_TAGS
@@ -260,7 +273,7 @@ class MetadataBox(QtWidgets.QTableWidget):
             lookup_func(v)
 
     def edit(self, index, trigger, event):
-        if index.column() != 2:
+        if index.column() != self.COLUMN_NEW:
             return False
         item = self.itemFromIndex(index)
         if item.flags() & QtCore.Qt.ItemIsEditable and \
@@ -278,49 +291,86 @@ class MetadataBox(QtWidgets.QTableWidget):
                 return super().edit(index, trigger, event)
         return False
 
-    def event(self, e):
+    def keyPressEvent(self, event):
+        if event.matches(QtGui.QKeySequence.Copy):
+            self.copy_value()
+        elif event.matches(QtGui.QKeySequence.Paste):
+            self.paste_value()
+        else:
+            super().keyPressEvent(event)
+
+    def copy_value(self):
         item = self.currentItem()
-        if (item and e.type() == QtCore.QEvent.KeyPress and e.modifiers() == QtCore.Qt.ControlModifier):
+        if item:
             column = item.column()
             tag = self.tag_diff.tag_names[item.row()]
-            if e.key() == QtCore.Qt.Key_C:
-                if column == 1:
-                    self.clipboard = list(self.tag_diff.orig[tag])
-                elif column == 2:
-                    self.clipboard = list(self.tag_diff.new[tag])
-            elif e.key() == QtCore.Qt.Key_V and column == 2 and tag != "~length":
-                self.set_tag_values(tag, list(self.clipboard))
-        return super().event(e)
+            value = None
+            if column == self.COLUMN_ORIG:
+                value = self.tag_diff.orig[tag]
+            elif column == self.COLUMN_NEW:
+                value = self.tag_diff.new[tag]
+            if tag == '~length':
+                value = [format_time(value or 0), ]
+            if value is not None:
+                self.tagger.clipboard().setText(MULTI_VALUED_JOINER.join(value))
+                self.clipboard = value
+
+    def paste_value(self):
+        item = self.currentItem()
+        if item:
+            column = item.column()
+            tag = self.tag_diff.tag_names[item.row()]
+            if column == self.COLUMN_NEW and self.tag_is_editable(tag):
+                self.set_tag_values(tag, self.clipboard)
+                self.update()
+
+    def update_clipboard(self):
+        clipboard = self.tagger.clipboard().text().split(MULTI_VALUED_JOINER)
+        if clipboard:
+            self.clipboard = clipboard
 
     def closeEditor(self, editor, hint):
         super().closeEditor(editor, hint)
         tag = self.tag_diff.tag_names[self.editing.row()]
         old = self.tag_diff.new[tag]
-        new = [editor.text()]
+        new = [self._get_editor_value(editor)]
         if old == new:
             self.editing.setText(old[0])
         else:
             self.set_tag_values(tag, new)
         self.editing = None
-        self.update()
+        self.update(drop_album_caches=tag == 'album')
+
+    @staticmethod
+    def _get_editor_value(editor):
+        if hasattr(editor, 'text'):
+            return editor.text()
+        elif hasattr(editor, 'toPlainText'):
+            return editor.toPlainText()
+        return ''
 
     def contextMenuEvent(self, event):
         menu = QtWidgets.QMenu(self)
         if self.objects:
-            tags = self.selected_tags(discard=('~length',))
-            if len(tags) == 1:
+            tags = self.selected_tags()
+            single_tag = len(tags) == 1
+            if single_tag:
                 selected_tag = tags[0]
+                editable = self.tag_is_editable(selected_tag)
                 edit_tag_action = QtWidgets.QAction(_("Edit..."), self.parent)
                 edit_tag_action.triggered.connect(partial(self.edit_tag, selected_tag))
                 edit_tag_action.setShortcut(self.edit_tag_shortcut.key())
+                edit_tag_action.setEnabled(editable)
                 menu.addAction(edit_tag_action)
                 if selected_tag not in self.preserved_tags:
                     add_to_preserved_tags_action = QtWidgets.QAction(_("Add to 'Preserve Tags' List"), self.parent)
                     add_to_preserved_tags_action.triggered.connect(partial(self.preserved_tags.add, selected_tag))
+                    add_to_preserved_tags_action.setEnabled(editable)
                     menu.addAction(add_to_preserved_tags_action)
                 else:
                     remove_from_preserved_tags_action = QtWidgets.QAction(_("Remove from 'Preserve Tags' List"), self.parent)
                     remove_from_preserved_tags_action.triggered.connect(partial(self.preserved_tags.discard, selected_tag))
+                    remove_from_preserved_tags_action.setEnabled(editable)
                     menu.addAction(remove_from_preserved_tags_action)
             removals = []
             useorigs = []
@@ -329,8 +379,8 @@ class MetadataBox(QtWidgets.QTableWidget):
                 column = item.column()
                 for tag in tags:
                     if tag in self.lookup_tags().keys():
-                        if (column == 1 or column == 2) and len(tags) == 1 and item.text():
-                            if column == 1:
+                        if (column == self.COLUMN_ORIG or column == self.COLUMN_NEW) and single_tag and item.text():
+                            if column == self.COLUMN_ORIG:
                                 values = self.tag_diff.orig[tag]
                             else:
                                 values = self.tag_diff.new[tag]
@@ -339,26 +389,50 @@ class MetadataBox(QtWidgets.QTableWidget):
                             menu.addAction(lookup_action)
                     if self.tag_is_removable(tag):
                         removals.append(partial(self.remove_tag, tag))
-                    status = self.tag_diff.status[tag] & TagStatus.Changed
-                    if status == TagStatus.Changed or status == TagStatus.Removed:
+                    status = self.tag_diff.status[tag] & TagStatus.CHANGED
+                    if status == TagStatus.CHANGED or status == TagStatus.REMOVED:
+                        file_tracks = []
+                        track_albums = set()
                         for file in self.files:
                             objects = [file]
-                            if file.parent in self.tracks and len(self.files & set(file.parent.linked_files)) == 1:
+                            if file.parent in self.tracks and len(self.files & set(file.parent.files)) == 1:
                                 objects.append(file.parent)
+                                file_tracks.append(file.parent)
+                                track_albums.add(file.parent.album)
                             orig_values = list(file.orig_metadata.getall(tag)) or [""]
                             useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
-                if removals:
-                    remove_tag_action = QtWidgets.QAction(_("Remove"), self.parent)
-                    remove_tag_action.triggered.connect(lambda: [f() for f in removals])
-                    remove_tag_action.setShortcut(self.remove_tag_shortcut.key())
-                    menu.addAction(remove_tag_action)
+                        for track in set(self.tracks)-set(file_tracks):
+                            objects = [track]
+                            orig_values = list(track.orig_metadata.getall(tag)) or [""]
+                            useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
+                            track_albums.add(track.album)
+                        for album in track_albums:
+                            objects = [album]
+                            orig_values = list(album.orig_metadata.getall(tag)) or [""]
+                            useorigs.append(partial(self.set_tag_values, tag, orig_values, objects))
+                remove_tag_action = QtWidgets.QAction(_("Remove"), self.parent)
+                remove_tag_action.triggered.connect(partial(self._apply_update_funcs, removals))
+                remove_tag_action.setShortcut(self.remove_tag_shortcut.key())
+                remove_tag_action.setEnabled(bool(removals))
+                menu.addAction(remove_tag_action)
                 if useorigs:
                     name = ngettext("Use Original Value", "Use Original Values", len(useorigs))
                     use_orig_value_action = QtWidgets.QAction(name, self.parent)
-                    use_orig_value_action.triggered.connect(lambda: [f() for f in useorigs])
+                    use_orig_value_action.triggered.connect(partial(self._apply_update_funcs, useorigs))
                     menu.addAction(use_orig_value_action)
                     menu.addSeparator()
-            if len(tags) == 1 or removals or useorigs:
+                if single_tag:
+                    menu.addSeparator()
+                    copy_action = QtWidgets.QAction(icontheme.lookup('edit-copy', icontheme.ICON_SIZE_MENU), _("&Copy"), self)
+                    copy_action.triggered.connect(self.copy_value)
+                    copy_action.setShortcut(QtGui.QKeySequence.Copy)
+                    menu.addAction(copy_action)
+                    paste_action = QtWidgets.QAction(icontheme.lookup('edit-paste', icontheme.ICON_SIZE_MENU), _("&Paste"), self)
+                    paste_action.triggered.connect(self.paste_value)
+                    paste_action.setShortcut(QtGui.QKeySequence.Paste)
+                    paste_action.setEnabled(editable)
+                    menu.addAction(paste_action)
+            if single_tag or removals or useorigs:
                 menu.addSeparator()
             menu.addAction(self.add_tag_action)
             menu.addSeparator()
@@ -366,11 +440,16 @@ class MetadataBox(QtWidgets.QTableWidget):
         menu.exec_(event.globalPos())
         event.accept()
 
+    def _apply_update_funcs(self, funcs):
+        for f in funcs:
+            f()
+        self.parent.update_selection(new_selection=False, drop_album_caches=True)
+
     def edit_tag(self, tag):
         EditTagDialog(self.parent, tag).exec_()
 
     def edit_selected_tag(self):
-        tags = self.selected_tags(discard=('~length',))
+        tags = self.selected_tags(filter_func=self.tag_is_editable)
         if len(tags) == 1:
             self.edit_tag(tags[0])
 
@@ -381,36 +460,39 @@ class MetadataBox(QtWidgets.QTableWidget):
     def set_tag_values(self, tag, values, objects=None):
         if objects is None:
             objects = self.objects
-        self.parent.ignore_selection_changes = True
-        if values == [""]:
-            values = []
-        if not values and self.tag_is_removable(tag):
-            for obj in objects:
-                obj.metadata.delete(tag)
-        elif values:
-            for obj in objects:
-                obj.metadata[tag] = values
-                obj.update()
-        self.update()
-        self.parent.ignore_selection_changes = False
+        with self.parent.ignore_selection_changes:
+            if values == [""]:
+                values = []
+            if not values and self.tag_is_removable(tag):
+                for obj in objects:
+                    del obj.metadata[tag]
+                    obj.update()
+            elif values:
+                for obj in objects:
+                    obj.metadata[tag] = values
+                    obj.update()
 
     def remove_tag(self, tag):
         self.set_tag_values(tag, [])
 
     def remove_selected_tags(self):
-        for tag in self.selected_tags(discard=('~length',)):
+        for tag in self.selected_tags(filter_func=self.tag_is_removable):
             if self.tag_is_removable(tag):
                 self.remove_tag(tag)
+        self.parent.update_selection(new_selection=False, drop_album_caches=True)
 
     def tag_is_removable(self, tag):
-        return self.tag_diff.status[tag] & TagStatus.NotRemovable == 0
+        return self.tag_diff.status[tag] & TagStatus.NOTREMOVABLE == 0
 
-    def selected_tags(self, discard=None):
-        if discard is None:
-            discard = set()
+    def tag_is_editable(self, tag):
+        return self.tag_diff.status[tag] & TagStatus.READONLY == 0
+
+    def selected_tags(self, filter_func=None):
         tags = set(self.tag_diff.tag_names[item.row()]
                    for item in self.selectedItems())
-        return list(tags.difference(discard))
+        if filter_func:
+            tags = filter(filter_func, tags)
+        return list(tags)
 
     def _update_selection(self):
         files = set()
@@ -421,7 +503,7 @@ class MetadataBox(QtWidgets.QTableWidget):
                 files.add(obj)
             elif isinstance(obj, Track):
                 tracks.add(obj)
-                files.update(obj.linked_files)
+                files.update(obj.files)
             elif isinstance(obj, Cluster) and obj.can_edit_tags():
                 objects.add(obj)
                 files.update(obj.files)
@@ -429,7 +511,7 @@ class MetadataBox(QtWidgets.QTableWidget):
                 objects.add(obj)
                 tracks.update(obj.tracks)
                 for track in obj.tracks:
-                    files.update(track.linked_files)
+                    files.update(track.files)
         objects.update(files)
         objects.update(tracks)
         self.selection_dirty = False
@@ -441,14 +523,16 @@ class MetadataBox(QtWidgets.QTableWidget):
         self.selection_mutex.unlock()
 
     @throttle(100)
-    def update(self):
+    def update(self, drop_album_caches=False):
         if self.editing:
             return
+        new_selection = self.selection_dirty
         if self.selection_dirty:
             self._update_selection()
-        thread.run_task(self._update_tags, self._update_items)
+        thread.run_task(partial(self._update_tags, new_selection, drop_album_caches), self._update_items,
+            thread_pool=self.tagger.priority_thread_pool)
 
-    def _update_tags(self):
+    def _update_tags(self, new_selection=True, drop_album_caches=False):
         self.selection_mutex.lock()
         files = self.files
         tracks = self.tracks
@@ -457,12 +541,34 @@ class MetadataBox(QtWidgets.QTableWidget):
         if not (files or tracks):
             return None
 
+        if new_selection or drop_album_caches:
+            self._single_file_album = len(set([file.metadata["album"] for file in files])) == 1
+            self._single_track_album = len(set([track.metadata["album"] for track in tracks])) == 1
+
+        while not new_selection:  # Just an if with multiple exit points
+            # If we are dealing with the same selection
+            # skip updates unless it we are dealing with a single file/track
+            if len(files) == 1:
+                break
+            if len(tracks) == 1:
+                break
+            # Or if we are dealing with a single cluster/album
+            if self._single_file_album:
+                break
+            if self._single_track_album:
+                break
+            return self.tag_diff
+
+        self.colors = {
+            TagStatus.NOCHANGE: self.palette().color(QtGui.QPalette.Text),
+            TagStatus.REMOVED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_removed')),
+            TagStatus.ADDED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_added')),
+            TagStatus.CHANGED: QtGui.QBrush(interface_colors.get_qcolor('tagstatus_changed'))
+        }
+
         tag_diff = TagDiff(max_length_diff=config.setting["ignore_track_duration_difference_under"])
         orig_tags = tag_diff.orig
         new_tags = tag_diff.new
-        # existing_tags are orig_tags that would not be overwritten by
-        # any new_tags, assuming clear_existing_tags is disabled.
-        existing_tags = set()
         tag_diff.objects = len(files)
 
         clear_existing_tags = config.setting["clear_existing_tags"]
@@ -476,31 +582,34 @@ class MetadataBox(QtWidgets.QTableWidget):
                 new_values = new_metadata.getall(name)
                 orig_values = orig_metadata.getall(name)
 
-                if not ((new_values and name not in existing_tags) or clear_existing_tags):
+                if not clear_existing_tags and not new_values:
                     new_values = list(orig_values or [""])
-                    existing_tags.add(name)
 
                 removed = name in new_metadata.deleted_tags
                 tag_diff.add(name, orig_values, new_values, True, removed)
 
-            tag_diff.add("~length",
-                         str(orig_metadata.length), str(new_metadata.length), False)
+            tag_diff.add("~length", str(orig_metadata.length), str(new_metadata.length),
+                         removable=False, readonly=True)
 
         for track in tracks:
             if track.num_linked_files == 0:
-                for name, values in track.metadata.rawitems():
+                for name, new_values in track.metadata.rawitems():
                     if not name.startswith("~"):
-                        tag_diff.add(name, values, values, True)
+                        if name in track.orig_metadata:
+                            orig_values = track.orig_metadata.getall(name)
+                        else:
+                            orig_values = new_values
+                        tag_diff.add(name, orig_values, new_values, True)
 
                 length = str(track.metadata.length)
-                tag_diff.add("~length", length, length, False)
+                tag_diff.add("~length", length, length, removable=False, readonly=True)
 
                 tag_diff.objects += 1
 
         all_tags = set(list(orig_tags.keys()) + list(new_tags.keys()))
-        tag_names = COMMON_TAGS + \
-            sorted(all_tags.difference(COMMON_TAGS),
-                   key=lambda x: display_tag_name(x).lower())
+        common_tags = [tag for tag in config.setting['metadatabox_top_tags'] if tag in all_tags]
+        tag_names = common_tags + sorted(all_tags.difference(common_tags),
+                                         key=lambda x: display_tag_name(x).lower())
 
         if config.persist["show_changes_first"]:
             tags_by_status = {}
@@ -508,13 +617,13 @@ class MetadataBox(QtWidgets.QTableWidget):
             for tag in tag_names:
                 tags_by_status.setdefault(tag_diff.tag_status(tag), []).append(tag)
 
-            for status in (TagStatus.Changed, TagStatus.Added,
-                           TagStatus.Removed, TagStatus.NoChange):
+            for status in (TagStatus.CHANGED, TagStatus.ADDED,
+                           TagStatus.REMOVED, TagStatus.NOCHANGE):
                 tag_diff.tag_names += tags_by_status.pop(status, [])
         else:
             tag_diff.tag_names = [
                 tag for tag in tag_names if
-                tag_diff.status[tag] != TagStatus.Empty]
+                tag_diff.status[tag] != TagStatus.EMPTY]
 
         return tag_diff
 
@@ -563,7 +672,7 @@ class MetadataBox(QtWidgets.QTableWidget):
             self.set_item_value(new_item, self.tag_diff.new, name)
 
             font = new_item.font()
-            if result.tag_status(name) == TagStatus.Removed:
+            if result.tag_status(name) == TagStatus.REMOVED:
                 font.setStrikeOut(True)
             else:
                 font.setStrikeOut(False)
@@ -571,12 +680,21 @@ class MetadataBox(QtWidgets.QTableWidget):
             new_item.setFont(font)
 
             color = self.colors.get(result.tag_status(name),
-                                    self.colors[TagStatus.NoChange])
+                                    self.colors[TagStatus.NOCHANGE])
             orig_item.setForeground(color)
             new_item.setForeground(color)
 
+            alignment = QtCore.Qt.AlignLeft | QtCore.Qt.AlignTop
+            tag_item.setTextAlignment(alignment)
+            orig_item.setTextAlignment(alignment)
+            new_item.setTextAlignment(alignment)
+
+            # Adjust row height to content size
+            self.setRowHeight(i, self.sizeHintForRow(i))
+
     def set_item_value(self, item, tags, name):
         text, italic = tags.display_value(name)
+        item.setData(QtCore.Qt.UserRole, name)
         item.setText(text)
         font = item.font()
         font.setItalic(italic)
