@@ -3,7 +3,7 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006 Lukáš Lalinský
-# Copyright (C) 2013 Philipp Wolfer
+# Copyright (C) 2013, 2020 Philipp Wolfer
 # Copyright (C) 2013, 2018 Laurent Monin
 # Copyright (C) 2016-2017 Sambhav Kothari
 #
@@ -42,13 +42,15 @@ class NetworkOptionsPage(OptionsPage):
 
     options = [
         config.BoolOption("setting", "use_proxy", False),
+        config.TextOption("setting", "proxy_type", "http"),
         config.TextOption("setting", "proxy_server_host", ""),
         config.IntOption("setting", "proxy_server_port", 80),
         config.TextOption("setting", "proxy_username", ""),
         config.TextOption("setting", "proxy_password", ""),
         config.BoolOption("setting", "browser_integration", True),
         config.IntOption("setting", "browser_integration_port", 8000),
-        config.BoolOption("setting", "browser_integration_localhost_only", True)
+        config.BoolOption("setting", "browser_integration_localhost_only", True),
+        config.IntOption("setting", "network_transfer_timeout_seconds", 30),
     ]
 
     def __init__(self, parent=None):
@@ -58,23 +60,34 @@ class NetworkOptionsPage(OptionsPage):
 
     def load(self):
         self.ui.web_proxy.setChecked(config.setting["use_proxy"])
+        if config.setting["proxy_type"] == 'socks':
+            self.ui.proxy_type_socks.setChecked(True)
+        else:
+            self.ui.proxy_type_http.setChecked(True)
         self.ui.server_host.setText(config.setting["proxy_server_host"])
         self.ui.server_port.setValue(config.setting["proxy_server_port"])
         self.ui.username.setText(config.setting["proxy_username"])
         self.ui.password.setText(config.setting["proxy_password"])
+        self.ui.transfer_timeout.setValue(config.setting["network_transfer_timeout_seconds"])
         self.ui.browser_integration.setChecked(config.setting["browser_integration"])
         self.ui.browser_integration_port.setValue(config.setting["browser_integration_port"])
         self.ui.browser_integration_localhost_only.setChecked(
             config.setting["browser_integration_localhost_only"])
-        self.ui.browser_integration_port.valueChanged.connect(self.change_browser_integration_port)
 
     def save(self):
         config.setting["use_proxy"] = self.ui.web_proxy.isChecked()
+        if self.ui.proxy_type_socks.isChecked():
+            config.setting["proxy_type"] = 'socks'
+        else:
+            config.setting["proxy_type"] = 'http'
         config.setting["proxy_server_host"] = self.ui.server_host.text()
         config.setting["proxy_server_port"] = self.ui.server_port.value()
         config.setting["proxy_username"] = self.ui.username.text()
         config.setting["proxy_password"] = self.ui.password.text()
         self.tagger.webservice.setup_proxy()
+        transfer_timeout = self.ui.transfer_timeout.value()
+        config.setting["network_transfer_timeout_seconds"] = transfer_timeout
+        self.tagger.webservice.set_transfer_timeout(transfer_timeout)
         config.setting["browser_integration"] = self.ui.browser_integration.isChecked()
         config.setting["browser_integration_port"] = self.ui.browser_integration_port.value()
         config.setting["browser_integration_localhost_only"] = \
@@ -87,8 +100,5 @@ class NetworkOptionsPage(OptionsPage):
         else:
             self.tagger.browser_integration.stop()
 
-    def change_browser_integration_port(self, port):
-        config.setting["browser_integration_port"] = port
-
 
 register_options_page(NetworkOptionsPage)
