@@ -217,6 +217,7 @@ class PluginsOptionsPage(OptionsPage):
     PARENT = None
     SORT_ORDER = 70
     ACTIVE = True
+    HELP_URL = '/config/options_plugins.html'
 
     options = [
         config.ListOption("setting", "enabled_plugins", []),
@@ -433,7 +434,7 @@ class PluginsOptionsPage(OptionsPage):
                 self,
                 _("Plugin '%s'") % plugin_name,
                 _("The plugin '%s' will be upgraded to version %s on next run of Picard.")
-                % (plugin.name, item.new_version)
+                % (plugin.name, item.new_version.to_string(short=True))
             )
 
             item.upgrade_to_version = item.new_version
@@ -588,17 +589,20 @@ class PluginsOptionsPage(OptionsPage):
                 label = _("Restart Picard to upgrade to new version")
             else:
                 label = _("New version available")
-            text.append("<b>" + label + ": " + item.new_version + "</b>")
+            version_str = item.new_version.to_string(short=True)
+            text.append("<b>{0}: {1}</b>".format(label, version_str))
         if plugin.description:
             text.append(plugin.description + "<hr width='90%'/>")
-        if plugin.name:
-            text.append("<b>" + _("Name") + "</b>: " + plugin.name)
-        if plugin.author:
-            text.append("<b>" + _("Authors") + "</b>: " + plugin.author)
-        if plugin.license:
-            text.append("<b>" + _("License") + "</b>: " + plugin.license)
-        text.append("<b>" + _("Files") + "</b>: " + plugin.files_list)
-        self.ui.details.setText("<p>%s</p>" % "<br/>\n".join(text))
+        infos = [
+            (_("Name"), plugin.name),
+            (_("Authors"), plugin.author),
+            (_("License"), plugin.license),
+            (_("Files"), plugin.files_list),
+        ]
+        for label, value in infos:
+            if value:
+                text.append("<b>{0}:</b> {1}".format(label, value))
+        self.ui.details.setText("<p>{0}</p>".format("<br/>\n".join(text)))
 
     def change_details(self):
         item = self.selected_item()
